function Q = computeGreshgorinWeights(xPoint,uPoint)
%COMPUTEGRESHGORINWEIGHTS Summary of this function goes here
% 
% [OUTPUTARGS] = COMPUTEGRESHGORINWEIGHTS(INPUTARGS) Explain usage here
% 
% Examples: 
% 
% Provide sample usage code here
% 
% See also: List related files here

% $Author: suwartad $	$Date: 2017/01/04 17:26:50 $	$Revision: 0.1 $
% Copyright: Process Control Group - NTNU Trondheim 2017

format long;
import casadi.*

%% parameter values
NT  = 41;                             % number of trays
LT  = 2.827;                          % Reflux
VB  = 3.454;                          % Boilup
F   = 1.0;                            % Feedrate
zF  = 1.0;
D   = 0.5;
B   = 0.5;
qF  = 1.0;                           % Feed liquid fraction

dist.F_0  = 0.3;
dist.NT   = NT;
dist.zF   = zF;
dist.qF   = qF;

% price setting
price.pf = 1; 
price.pV = 0.02;
price.pB = 2; 
price.pD = 0;

% Symbolic primitives
x = {};
l = {};
for i=1:2*NT+2       % +2 due to CSTR
   x{i} = SX.sym(['x_' num2str(i)],1);
   l{i} = SX.sym(['l_' num2str(i)],1);
end
u1  = SX.sym('u1');   % LT
u2  = SX.sym('u2');   % VB
u3  = SX.sym('u3');   % F
u4  = SX.sym('u3');   % D
u5  = SX.sym('u4');   % B
% concatenate states and controls 
x   = vertcat(x{:});
x   = [x;u1;u2;u3;u4;u5];

% define the dynamics as equality constraints and additional inequality
% constraints
% lbx, ubx, lbg, and ubg
[obj,eq, ~, ~, ~, ~] = buildModelEq(x,dist,price);

%lamda
load LamdaCstrDist.mat lamda;

%% Compute Hessian and perform Greshgorin convexification
xsol = [xPoint;uPoint];
lambda.eqnonlin = lamda;
% extract Lagrange multiplier
l   = vertcat(l{:});
L   = obj + l'*eq;

Lagr = Function('Lagr', {x,l}, {L}, char('x','l'), char('Lagr'));
H    = Function(Lagr.hessian('x','Lagr'));
Hx   = H(xsol,lambda.eqnonlin);
Hx   = full(Hx);
Q    = Greshgorin(Hx);

save Q.mat Q;
end

function [J, ceq, lbx, ubx, lbg, ubg] = buildModelEq(u,dist,price)
import casadi.* 
NT = dist.NT;
% Location of feed stage (stages are counted from the bottom):
NF = 21;
% Relative volatility
alpha = 1.5;
% Nominal liquid holdups
% M0(1)=0.5;      	% Nominal reboiler holdup (kmol)
% i=2:NT-1; M0(i)=0.5*ones(1,NT-2);% Nominal stage (tray) holdups (kmol)
% M0(NT)=0.5;      	% Nominal condenser holdup (kmol)
Muw   = 0.5;
% Data for linearized liquid flow dynamics (does not apply to reboiler and condenser):
taul = 0.063;     	% time constant for liquid dynamics (min)
F0   = 1;	 	    % Nominal feed rate (kmol/min) 
qF0  = 1; 		    % Nominal fraction of liquid in feed 
L0   = 2.70629;     % Nominal reflux flow (from steady-state data)
L0b  = L0 + qF0*F0;	% Nominal liquid flow below feed (kmol/min)
%lambda = 0;		    % Effect of vapor flow on liquid flow ("K2-effect")

% % P-Controllers for control of reboiler and condenser hold up.
% % controller gains
% KcB = 10;  
% KcD = 10;         
% % Nominal holdups - these are rather small
% MDs = 0.5; 
% MBs = 0.5;          
% % Nominal flows
% Ds = 0.5; 
% Bs = 0.5;          
% 
% % Actual reboiler and condenser holdup
% MB = u(NT+2);  
% MD = u(2*NT+1); 
% D  = Ds +(MD-MDs)*KcD;       % Distillate flow
% B  = Bs +(MB-MBs)*KcB;       % Bottoms flow    
D   = u(2*NT+6);
B   = u(2*NT+7);


% Inputs and disturbances
%LT = U(1);                            % Reflux
%LT = u(2*NT+1);
LT = u(2*NT+3);
%LT = u(2*NT+3)/sf1;
%VB = U(2);                            % Boilup
%VB = u(2*NT+2);
VB = u(2*NT+4);
%VB = u(2*NT+4)/sf2;
%D  = U(3);                            % Distillate
%B  = U(4);                            % Bottoms

%F  = U(5);                            % Feedrate
%F   = u(2*NT+5)/sf3;
F   = u(2*NT+5);
F_0  = dist.F_0;
% F_0  = u(2*NT+5);
%zF = U(6);                            % Feed composition
zF = dist.zF;
%qF = U(7);                            % Feed liquid fraction
qF = dist.qF;



% THE MODEL
% objective function  
%F = F_0 + D;
%F_0 = F - D;
% F_0 = 0.6; 
%J = price.pf*F + price.pV*VB - price.pB*B - price.pD*D;
J = price.pf*F_0 + price.pV*VB - price.pB*B - price.pD*D;
%J = price.pf*F_0 + price.pV*u(2*NT+4) - price.pB*B - price.pD*D;

% Vapor-liquid equilibria
%i=1:NT-1;    y(i)=alpha*u(i)./(1+(alpha-1)*u(i));
for i=1:NT-1
   y{i}  = SX.sym(['y_' num2str(i)],1);
   V{i}  = SX.sym(['V_' num2str(i)],1);
   L{i}  = SX.sym(['L_' num2str(i)],1);
   dMdt{i}  = SX.sym(['dMdt_' num2str(i)],1);
   dMxdt{i} = SX.sym(['dMxdt_' num2str(i)],1);
end
L{NT}     = SX.sym(['L_' num2str(NT)],1);
dMdt{NT}  = SX.sym(['dMdt_' num2str(NT)],1);
dMxdt{NT} = SX.sym(['dMxdt_' num2str(NT)],1);
dMdt{NT+1}  = SX.sym(['dMdt_' num2str(NT+1)],1);
dMxdt{NT+1} = SX.sym(['dMxdt_' num2str(NT+1)],1);

y = vertcat(y{:});
for i=1:NT-1
    y(i)=alpha*u(i)/(1+(alpha-1)*u(i));
end

% Vapor Flows assuming constant molar flows
V = vertcat(V{:});
for i=1:NT-1
    %V(i) = VB;
    if i >= NF
        V(i) = VB + (1-qF)*F;
    else
        V(i) = VB;
    end
end


L = vertcat(L{:});
L(NT)=LT;
for i=2:NT-1
    if i <=NF
        L(i) = L0b + (u(NT+1+i)-Muw)./taul;
    else
        L(i) = L0  + (u(NT+1+i)-Muw)./taul;
    end
end



% Time derivatives from  material balances for 
% 1) total holdup and 2) component holdup

% Column
%i=2:NT-1;
%dMdt(i) = L(i+1)         - L(i)       + V(i-1)         - V(i);
%dMxdt(i)= L(i+1).*u(i+1) - L(i).*u(i) + V(i-1).*y(i-1) - V(i).*y(i);
dMdt  = vertcat(dMdt{:});
dMxdt = vertcat(dMxdt{:});
for i=2:NT-1
    dMdt(i) = L(i+1)         - L(i)       + V(i-1)         - V(i);
    dMxdt(i)= L(i+1).*u(i+1,1) - L(i).*u(i,1) + V(i-1).*y(i-1) - V(i).*y(i);
end

% Correction for feed at the feed stage
% The feed is assumed to be mixed into the feed stage
dMdt(NF) = dMdt(NF)  + F;
%dMxdt(NF)= dMxdt(NF) + F*zF;
dMxdt(NF)= dMxdt(NF) + F*u(NT+1);

% Reboiler (assumed to be an equilibrium stage)
dMdt(1) = L(2)      - V(1)      - B;
dMxdt(1)= L(2)*u(2) - V(1)*y(1) - B*u(1);

% Total condenser (no equilibrium stage)
dMdt(NT) = V(NT-1)         - LT       - D;
dMxdt(NT)= V(NT-1)*y(NT-1) - LT*u(NT) - D*u(NT);

% Compute the derivative for the mole fractions from d(Mx) = x dM + M dx
%i=1:NT;
%ceq(i) = (dMxdt(i) - u(i).*dMdt(i) )./u(NT+i,1);
%ceq(i) = (dMxdt(i) - u(i).*dMdt(i) )./M(i);
%dxdt(i) = (dMxdt(i) - x(i).*dMdt(i) )./M(i);
%for i=1:(2*NT+4)  % add CSTR
for i=1:(2*NT+2)
    ceq{i}  = SX.sym(['ceq_' num2str(i)],1);
end


% CSTR model
% k1          = 0.341/60.0; %1e8*np.exp(-6.67e4/(8.14*T_R))
k1          = 34.1/60.0;
dMdt(NT+1)  = F_0 + D - F;
dMxdt(NT+1) = F_0*zF + D*u(NT) - F*u(NT+1) - k1*u(2*NT+2)*u(NT+1);


ceq  = vertcat(ceq{:});
for i=1:NT+1
    %ceq(i) = (dMxdt(i) - u(i).*dMdt(i) ) / u(NT+1+i,1);
    ceq(i) = dMxdt(i);
end

% load measurement noise
%load noise1pct.mat;
for i=1:NT+1
    ceq(NT+1+i) = dMdt(i);
    %ceq(NT+1+i) = dMdt(i) + noise(i);
end

% bound constraints
% lb_u = [0.1; 0.1];
% ub_u = [10; 4.008];
% after scaling LT and VB
% lb_u = [0; 0];
% ub_u = [1; 1];
% lb_u = [0; 0; 0];
% ub_u = [1; 1; 1];
% lb_u = [0.1; 0.1; 0.1];
% ub_u = [10; 4.008; 10.0];
lb_u = [0.1; 0.1; 0.1; 0.1; 0.1];
%lb_u = [0.1; 2.5; 0.1; 0.1; 0.1];
ub_u = [10; 4.008; 10; 1.0; 1.0];
%ub_u = [10; 4.008; 10; 10; 10];
%ub_u = ones(5,1);

% State bounds and initial guess
x_min     = zeros(84,1);
x_max     = ones(84,1);
xB_max    = 0.1;
%xB_max    = 0.08;
%xB_max    = 0.05;
xD_min    = 0.99;
x_max(1)  = xB_max;
%x_min(1)  = 0.001;
%x_min(41) = xD_min;
% x_min(43) = 0.1;
% x_max(43) = 0.9;
% x_min(83) = 0.1;
% x_max(83) = 0.9;
x_min(84) = 0.3;
x_max(84) = 0.7;
lbx  = [x_min;lb_u];
ubx  = [x_max;ub_u];
lbg  = zeros(2*NT+2,1);
ubg  = zeros(2*NT+2,1);
% lb_u = [0.1; 0.1; 0.01];
% ub_u = [10; 4.008; 2.0];
% lb_x = 1e-3*ones(2*NT+2,1);
% ub_x = ones(2*NT+2,1);
% lbx  = [lb_x;lb_u];
% ubx  = [ub_x;ub_u];
% lbg  = zeros(2*NT+2,1);
% ubg  = zeros(2*NT+2,1);

% % inequality constraints
% xB = u(1); 
% xD = u(NT);
% ceq(2*NT+3) = xB;
% lbg(2*NT+3) = 0;
% ubg(2*NT+3) = 0.1;
% ceq(2*NT+4) = xD;
% lbg(2*NT+4) = 0.95;
% ubg(2*NT+4) = 1.0;

end

function Q = Greshgorin(H)
numH    = size(H,1);
Q       = zeros(numH,numH);
%Q       = eye(numH);
%delta   = 1e-1;
%delta   = 1e-2;
%delta   = 2.5e-1;  % normal case
%delta   = 0.5;
delta   = 1;
%delta   = 1.5;
%delta   = 2;
%delta   = 2.5;      % with measurement noise 1 percent
%delta   = 5;
%delta   = 100;
%delta   = 1e-6;
for i=1:numH  % iterate all row of Hessian
    sumRow = 0;
    for j=1:numH
        if j ~= i
            sumRow = sumRow + abs(H(i,j));
        end
    end
    %sumRow = sumRow - abs(H(i,i));
    
    if H(i,i) <= sumRow   % include equality 
    %if H(i,i) < sumRow 
        %Q(i,i) = sumRow - H(i,i) + delta;
        Q(i,i) = sumRow - H(i,i) + delta;
    end
end

% % loop over Qm to obtain maximum number
% for i=1:numH
%     if Q(i,i) > Qm(i,1)
%         Qm(i,1) = Q(i,i);
%     end
% end
Q = diag(Q);
end